@@ -16,6 +16,14 @@ from .error import (
     AsusRouterSSLError,
     AsusRouterValueError,
 )
-from .dataclass import AsusDevice, ConnectedDevice, Key, Monitor
+from .dataclass import (
+    AiMeshDevice,
+    AsusDevice,
+    ConnectedDevice,
+    FilterDevice,
+    Firmware,
+    Key,
+    Monitor,
+)
 from .connection import Connection
 from .asusrouter import AsusRouter
