@@ -6,7 +6,6 @@ from typing import Any
 
 from asusrouter.const import (
     AR_DEFAULT_OVPN_CLIENTS,
-    AR_HOOK_TEMPLATE,
     AR_KEY_OVPN,
     AR_KEY_PARENTAL_CONTROL_MAC,
     AR_KEY_PARENTAL_CONTROL_NAME,
@@ -16,16 +15,25 @@ from asusrouter.const import (
     AR_MAP_PARENTAL_CONTROL_STATE,
     AR_MAP_RGB,
     AR_VPN_STATUS,
+    ENDHOOKS,
+    ENDPOINT,
+    ENDPOINT_ARGS,
     ERROR_VALUE_TYPE,
-    KEY_NVRAM_GET,
+    GWLAN,
+    HOOK,
     KEY_VPN,
+    MAP_NVRAM,
+    NVRAM_GET,
     PARAM_ERRNO,
     PARAM_STATE,
     PARAM_STATUS,
     PARAM_UNKNOWN,
+    RANGE_GWLAN,
+    WLAN,
+    WLAN_TYPE,
 )
 from asusrouter import FilterDevice
-from asusrouter.dataclass import ConnectedDevice
+from asusrouter.dataclass import AsusDevice, ConnectedDevice
 from asusrouter.error import AsusRouterValueError
 from .converters import int_from_str
 
@@ -35,24 +43,24 @@ def hook(commands: dict[str, str] | None = None) -> str:
 
     data = str()
     if commands is not None:
-        for item in commands:
-            data += AR_HOOK_TEMPLATE.format(item, commands[item])
+        for item, value in commands.items():
+            data += f"{item}({value});"
 
     return data
 
 
 def nvram(values: list[str] | str | None = None) -> str:
-    """NVRAM request compiler"""
+    """NVRAM compiler"""
 
     if values is None:
         return str()
 
-    if type(values) == str:
-        return AR_HOOK_TEMPLATE.format(KEY_NVRAM_GET, values)
+    if isinstance(values, str):
+        return f"{NVRAM_GET}({values});"
 
     request = str()
     for value in values:
-        request += AR_HOOK_TEMPLATE.format(KEY_NVRAM_GET, value)
+        request += f"{NVRAM_GET}({value});"
 
     return request
 
@@ -65,9 +73,9 @@ def rgb(raw: dict[int, dict[str, int]]) -> str:
     raw = dict(sorted(raw.items()))
 
     for led in raw:
-        for channel in AR_MAP_RGB:
-            if AR_MAP_RGB[channel] in raw[led]:
-                value += f"{raw[led][AR_MAP_RGB[channel]]},"
+        for code in AR_MAP_RGB.values():
+            if code in raw[led]:
+                value += f"{raw[led][code]},"
             else:
                 value += "0,"
 
@@ -81,17 +89,17 @@ def vpn_from_devicemap(
 ) -> dict[str, Any]:
     """Compile devicemap into VPN"""
 
-    if type(devicemap) != dict:
+    if not isinstance(devicemap, dict):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(devicemap, type(devicemap)))
 
-    if type(vpn) != dict:
-        vpn = dict()
+    if not isinstance(vpn, dict):
+        vpn = {}
 
     if KEY_VPN in devicemap:
         for num in range(1, AR_DEFAULT_OVPN_CLIENTS + 1):
             key = f"{AR_KEY_VPN_CLIENT}{num}"
             if not key in vpn:
-                vpn[key] = dict()
+                vpn[key] = {}
             if (
                 AR_KEY_OVPN.format(AR_KEY_VPN_CLIENT, num, PARAM_STATE)
                 in devicemap[KEY_VPN]
@@ -127,10 +135,10 @@ def vpn_from_devicemap(
 def parental_control(data: dict[str, FilterDevice]) -> dict[str, str]:
     """Compile parental control rules"""
 
-    if type(data) != dict:
+    if not isinstance(data, dict):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(data, type(data)))
 
-    state_lib = dict()
+    state_lib = {}
     for index, state in AR_MAP_PARENTAL_CONTROL_STATE.items():
         state_lib[state] = index
 
@@ -171,3 +179,55 @@ def connected_device(
     values.update(state)
 
     return ConnectedDevice(**values)
+
+
+def endpoint(endpoint: str, device: AsusDevice | dict[str, Any] | None = None) -> str:
+    """Compile endpoint string with required arguments"""
+
+    address = ENDPOINT.get(endpoint)
+    if not address:
+        address = ENDPOINT[HOOK] if endpoint in ENDHOOKS else str()
+
+    if isinstance(device, AsusDevice):
+        device = asdict(device)
+    if not isinstance(device, dict):
+        return address
+
+    address += "?"
+
+    if endpoint in ENDPOINT_ARGS:
+        for key in ENDPOINT_ARGS[endpoint]:
+            address += f"{ENDPOINT_ARGS[endpoint][key]}={device.get(key, str())}&"
+
+    return address
+
+
+def update_rec(left: dict[str, Any], right: dict[str, Any] | None = None) -> None:
+    """Update dictionary with values of other dictionary"""
+
+    if not right:
+        right = {}
+    for key, value in right.items():
+        if key in left and isinstance(left[key], dict) and isinstance(value, dict):
+            update_rec(left[key], value)
+        else:
+            left[key] = value
+
+
+def monitor_arg_nvram(wlan: list[str] | None) -> str | None:
+    """Compile NVRAM monitor"""
+
+    if not wlan:
+        return None
+
+    request = list()
+    for intf in wlan:
+        interface = WLAN_TYPE.get(intf)
+        for key in MAP_NVRAM[WLAN]:
+            request.append(key.value.format(interface))
+
+        for key in MAP_NVRAM[GWLAN]:
+            for gid in RANGE_GWLAN:
+                request.append(key.value.format(f"{interface}.{gid}"))
+
+    return nvram(request)
